@@ -10,6 +10,7 @@ from trytond.model import ModelView, ModelSQL, fields
 from trytond.model.exceptions import AccessError
 from trytond.wizard import Wizard, StateView, StateTransition, Button
 from trytond.pyson import If, In, Eval, Bool
+from trytond.tools import sortable_values
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
@@ -57,6 +58,8 @@ class PurchaseRequest(ModelSQL, ModelView):
         readonly=True)
     purchase_date = fields.Date('Best Purchase Date', readonly=True)
     supply_date = fields.Date('Expected Supply Date', readonly=True)
+    default_uom = fields.Function(fields.Many2One(
+            'product.uom', "Default UOM"), 'on_change_with_default_uom')
     default_uom_digits = fields.Function(fields.Integer('Default UOM Digits'),
         'on_change_with_default_uom_digits')
     stock_level = fields.Float('Stock at Supply Date', readonly=True,
@@ -84,12 +87,12 @@ class PurchaseRequest(ModelSQL, ModelView):
     exception_ignored = fields.Boolean('Ignored Exception')
 
     purchased_by = employee_field(
-        "Purchased By", states=['purchased', 'done', 'cancel', 'exception'])
+        "Purchased By", states=['purchased', 'done', 'cancelled', 'exception'])
     state = fields.Selection([
             ('purchased', "Purchased"),
             ('done', "Done"),
             ('draft', "Draft"),
-            ('cancel', "Cancel"),
+            ('cancelled', "Cancelled"),
             ('exception', "Exception"),
             ], "State", required=True, readonly=True, select=True)
 
@@ -164,6 +167,11 @@ class PurchaseRequest(ModelSQL, ModelView):
         # Migration from 4.4: remove required on origin
         tablehandler.not_null_action('origin', action='remove')
 
+        # Migration from 5.6: rename state cancel to cancelled
+        cursor.execute(*request.update(
+                [request.state], ['cancelled'],
+                where=request.state == 'cancel'))
+
     def get_rec_name(self, name):
         pool = Pool()
         Lang = pool.get('ir.lang')
@@ -219,11 +227,11 @@ class PurchaseRequest(ModelSQL, ModelView):
 
     def get_state(self):
         if self.purchase_line:
-            if (self.purchase_line.purchase.state == 'cancel'
+            if (self.purchase_line.purchase.state == 'cancelled'
                     and not self.exception_ignored):
                 return 'exception'
-            elif self.purchase_line.purchase.state == 'cancel':
-                return 'cancel'
+            elif self.purchase_line.purchase.state == 'cancelled':
+                return 'cancelled'
             elif self.purchase_line.purchase.state == 'done':
                 return 'done'
             else:
@@ -262,6 +270,11 @@ class PurchaseRequest(ModelSQL, ModelView):
         if self.product:
             return self.product.default_uom_category.id
 
+    @fields.depends('product')
+    def on_change_with_default_uom(self, name=None):
+        if self.product:
+            return self.product.default_uom.id
+
     @fields.depends('product')
     def on_change_with_default_uom_digits(self, name=None):
         if self.product:
@@ -285,7 +298,7 @@ class PurchaseRequest(ModelSQL, ModelView):
     @classmethod
     def view_attributes(cls):
         return [
-            ('/tree', 'visual', If(Eval('state') == 'cancel', 'muted', '')),
+            ('/tree', 'visual', If(Eval('state') == 'cancelled', 'muted', '')),
             ]
 
     @classmethod
@@ -313,8 +326,6 @@ class PurchaseRequest(ModelSQL, ModelView):
         Date = pool.get('ir.date')
         today = Date.today()
         for product_supplier in product.product_suppliers_used(**pattern):
-            if date is None:
-                return product_supplier
             supply_date = product_supplier.compute_supply_date(date=today)
             timedelta = date - supply_date
             if timedelta >= datetime.timedelta(0):
@@ -365,9 +376,7 @@ class CreatePurchase(Wizard):
             ])
 
     def default_ask_party(self, fields):
-        Request = Pool().get('purchase.request')
-        requests = Request.browse(Transaction().context['active_ids'])
-        for request in requests:
+        for request in self.records:
             if request.purchase_line:
                 continue
             if not request.party:
@@ -411,7 +420,7 @@ class CreatePurchase(Wizard):
         Line = pool.get('purchase.line')
         Date = pool.get('ir.date')
 
-        requests = Request.browse(Transaction().context['active_ids'])
+        requests = self.records
 
         if (getattr(self.ask_party, 'party', None)
                 and getattr(self.ask_party, 'company', None)):
@@ -445,7 +454,7 @@ class CreatePurchase(Wizard):
         requests = [r for r in requests if not r.purchase_line]
 
         keyfunc = partial(self._group_purchase_key, requests)
-        requests = sorted(requests, key=keyfunc)
+        requests = sorted(requests, key=sortable_values(keyfunc))
 
         purchases = []
         lines = []
@@ -510,23 +519,15 @@ class HandlePurchaseCancellationException(Wizard):
     cancel_request = StateTransition()
 
     def transition_reset(self):
-        pool = Pool()
-        PurchaseRequest = pool.get('purchase.request')
-
-        requests = PurchaseRequest.browse(Transaction().context['active_ids'])
-        for request in requests:
+        for request in self.records:
             request.purchase_line = None
-        PurchaseRequest.reset_purchased(requests)
+        self.model.reset_purchased(self.records)
         return 'end'
 
     def transition_cancel_request(self):
-        pool = Pool()
-        PurchaseRequest = pool.get('purchase.request')
-
-        requests = PurchaseRequest.browse(Transaction().context['active_ids'])
-        for request in requests:
+        for request in self.records:
             request.exception_ignored = True
-        PurchaseRequest.update_state(requests)
+        self.model.update_state(self.records)
         return 'end'
 
 
